package apt;

/**
 *
 * The ServerGateway class is the client-side
 * interface to the Apartment project server.
 *
 * It contains methods for the three main server tasks:
 *
 * --Reading the index of apartments.
 * --Reading an individual apartment.
 * --Saving an apartment.
 *
 */
public interface ServerGateway
{
    /**
     *
     * Read apartment summaries from server.
     * This will return a list of Apartment objects,
     * but without the words they contain.
     *
     * @return Vector containing a list of Apartment objects.
     *
     */
    public java.util.Vector readIndex() throws Exception;
    
    /**
     *
     * Load an apartment given its key.
     * Typically will try a local cache before loading from server.
     * Returns null if can't load or can't find
     * an apartment for this key.
     *
     * @param key the key designating the apartment to be loaded.
     *
     */
    public Apartment readApartment(Object key);
    
    /**
     *
     * Save an apartment.
     *
     * @param apartment the Apartment object to be saved.
     *
     */
    public void saveApartment(Apartment apartment) throws Exception;
}