package apt.plan2d;

import apt.Display;
import english.words.*;
import java.awt.*;
import java.util.*;
import java.applet.*;

/**
 *
 * This class represents a word floating around an apartment.
 * It has display information, position and velocity data,
 * and also the ability to snap onto and follow another word.
 *
 */
class FloatingWord implements apt.TypedWord
{
    static AudioClip clickSound;
    String word="?";
    double x,y, dx, dy;
    Font font;
    int width, height, wordSpacing;
    double speed;
    long age;
    long trackStart;
    Color color=Color.green;
    int type;
    Word englishWord;
    FloatingWord target;
    FloatingWord tracker;
    boolean snapped=false;
    static FloatingWord lastTracker, lastTarget;
    private Color snapColor;
    private String rootWord;

    FloatingWord(String word, String rootWord, Font font, int x, int y, int type)
    {
        this.word=word;
        this.rootWord=rootWord;
        this.font=font;
        this.type=type;
        this.x=x; this.y=y;
        FontMetrics measure=Toolkit.getDefaultToolkit().getFontMetrics(font);
        width=measure.stringWidth(word);
        height=measure.getAscent();
        wordSpacing=measure.stringWidth(" ");
        snapColor=Display.getDefaultDisplay().snapColor;
    }
    
    public String getWord()
    {
        return rootWord;
    }
    
    public int getType()
    {
        return type;
    }
    
    public boolean isAttached()
    {
        return tracker!=null || target!=null || tracker!=null;
    }

    public boolean track(FloatingWord target)
    {
        if (this.target==target) return false;
        long time=System.currentTimeMillis();
        if (time-trackStart<1000*10 || time-target.trackStart<1000*10)
            return false;
        if (target.tracker!=null) target.tracker.untrack();
        this.target=target;
        target.tracker=this;
        trackStart=time+(int)(1000*10*(Math.random()-.5));
        target.trackStart=trackStart;
        lastTracker=this;
        lastTarget=target;
        return true;
    }

    public void untrack()
    {
        //tracker=null;
        if (target!=null)
            target.tracker=null;
        target=null;
        snapped=false;
    }

    public void setEnglishWord(Word englishWord)
    {
        this.englishWord=englishWord;
    }

    public void setColor(Color color)
    {
        this.color=color;
    }

    public void setPosition(double x, double y)
    {
        this.x=x; this.y=y;
    }

    public void halt()
    {
        dx=0; dy=0;
    }

    public final double targetX()
    {
        return target.x-target.width/2-width/2-wordSpacing;
    }

    public void move()
    {
        if (target!=null)
        {
            if (snapped)
            {
                snapToTarget();
                return;
            }
            double ax=targetX();
            double ay=target.y;
            x=.6*x+.4*ax;
            y=.6*y+.4*ay;
            return;
        }



        double a=.3+170.0/(100+Math.sqrt(age));
        x+=a*dx; y+=a*dy;
    }

    public void snapToTarget()
    {
        x=targetX();
        y=target.y;
        if (!snapped && clickSound!=null)
            clickSound.play();
        snapped=true;
    }

    public boolean closeToTarget()
    {
        return target!=null &&
            (closeButNotEqual(target.y,y) ||
             closeButNotEqual(targetX(),x));
    }

    private boolean closeButNotEqual(double a, double b)
    {
        return Math.abs(a-b)<5 && a!=b;
    }

    public void aim(double ax, double ay, boolean exact)
    {
        aim(ax, ay, .02, exact ? 0 : 1);
    }

    public void aim(double ax, double ay, double speed, double rand)
    {
        dx*=.8; dy*=.8;
        dx+=speed*(ax-x)+rand*(Math.random()-.5);
        dy+=speed*(ay-y)+rand*(Math.random()-.5);
    }
    
    public double speed()
    {
        return Math.sqrt(dx*dx+dy*dy);
    }

    public void slow()
    {
        double s=speed();
        if (s>1)
        {
            dx*=.95;
            dy*=.95;
        }
    }


    public boolean inside(double ax, double ay, double aw, double ah)
    {
        return x>ax && x<ax+aw && y>ay && y<ay+ah;
    }

    public void draw(Graphics g)
    {
        g.setFont(font);
        g.setColor(this==lastTracker || this==lastTarget ? snapColor : color);
        double cx=x-width/2, cy=y+height/2;
        g.drawString(word, (int)cx, (int)cy);
    }

    public void makeOlder()
    {
        age++;
    }

    public long getAge()
    {
        return age;
    }

    public double distance(FloatingWord f)
    {
        double u=x-f.x;
        double v=(y-f.y)*4;
        return Math.sqrt(u*u+v*v);
    }

    public String toString()
    {
        return word+" ["+super.toString()+"]";
    }

}