package apt.plan2d;

import java.awt.*;

/**
 *
 * Hold the code for the motions for the
 * various rooms. This code is currently called
 * only by the ApartmentConstructor class.
 *
 */
class Motion
{
    static void implode(FloatingWord word, Room r, int j, double n, long time)
    {
        double s=2*Math.PI*j/n;
        double d=r.w/2-(time*.06+73*j)%(r.w/2);
        double ax=r.x+r.w/2+d*Math.cos(s);
        double ay=r.y+r.h/2+d*Math.sin(s);
        word.aim(ax, ay, true);
    }
    
    static void burst(FloatingWord word, Room r, int j, double n, long time)
    {
        double s=2*Math.PI*j/n;
        double q=40;
        double d=(time*.06+73*j)%(r.w/2+q);
        d-=q; if (d<0) d=0;
        double ax=r.x+r.w/2+d*Math.cos(s);
        double ay=r.y+r.h/2+d*Math.sin(s);
        word.aim(ax, ay, true);
    }
    
    static void jumble(FloatingWord word, Room r, int j, int n)
    {
        double x=r.x+.33*r.w, y=r.y+.7*r.h, w=.33*r.w, h=25;
        if (word.inside(x,y,w,h))
        {
            word.slow(); 
        }
        else
            word.aim(x+w/2, y+h/2, .01, .3);
    }
    
    static void boil(FloatingWord word, Room r, int j, int n, long time)
    {
        int side=1+(int)Math.sqrt(n-1);
        if (side==0) return;
        double ax=r.x+(j%side+((j/side)%2==0 ? .75 : .25))*r.w/side;
        double ay=r.y+.5*r.h*(1+(j/side+.5)/side);
        long t=time/1000;
        boolean bubble=t%3==j%3;
        if (bubble) ay-=10;
        word.aim(ax, ay, !bubble);
    }
    
    static void staticGrid(FloatingWord word, Room r, int j, int n)
    {
        int side=1+(int)Math.sqrt(n-1);
        if (side==0) return;
        double ax=r.x+(j%side+((j/side)%2==0 ? .75 : .25))*r.w/side;
        double ay=r.y+.5*r.h*(1+(j/side+.5)/side);
        word.aim(ax, ay, true);
    }
    
    static void align(FloatingWord word, Room r, int j, int n, long time)
    {
        int side=(int)(r.w/70)+1;
        double ax=r.x+(j%side+.5)*r.w/side;
        double s=Math.sin(time*.0003+(Math.PI*(j-j%side)/(double)n));
        double ay=r.y+(r.h-14*side)*s*s+14*(j%side);
        word.aim(ax, ay, true);
    }
    
    static void grid(FloatingWord word, Room r, int j, int n)
    {
        int side=1+(int)Math.sqrt(n-1);
        if (side==0) return;
        double ax=r.x+(j%side+.5)*r.w/side;
        double ay=r.y+(j/side+.5)*r.h/side;
        word.aim(ax, ay, false);
    }
    
    static void vertical(FloatingWord word, Room r, int j, int n, long time)
    {
        double h=r.h;
        if (h<2) return;
        double p=j/(double)n;
        double ax=r.x+r.w/2;
        double ay=r.y+(p*r.h+time*.03)%r.h;
        word.aim(ax, ay, true);
    }
    
    static void circle(FloatingWord word, Room r, double j, double n, long time)
    {
        double speed=4;
        double peri=speed*(r.w+r.h);
        double t=.3*time/peri;
        double s=2*Math.PI*j/n;
        double a=Math.cos(t+s);
        double b=Math.sin(t+s);
        a=.5+a*.3; b=.5+b*.3;
        word.aim(r.x+a*r.w, r.y+b*r.h, true);
    }
    
    static void sinuous(FloatingWord word, Room r, int j, int n, double p, double q, long time)
    {
        double speed=4;
        double peri=speed*(r.w+r.h);
        double t=.3*time/peri;
        double s=.35*j/speed;
        double a=Math.cos(q*(t+s));
        double b=Math.sin(p*(t+s));
        a=(1+a)/2; b=(1+b)/2;
        word.aim(r.x+a*r.w, r.y+b*r.h, true);
    }
    
    static void perspective(FloatingWord word, Room r, double j, double n, long time)
    {
        
        int side=1+(int)Math.sqrt(n-1);
        if (side==0) return;
        double ay=r.y+((j+.5)/(n+1))*r.h;
        
        double speed=4;
        double peri=speed*(r.w+r.h);
        double t=time/peri;
        double s=2*Math.PI*j/n;
        double a=Math.cos(t*(j/side+1)+s+j*j);
        a=.5+a*.3; 
        double ax=r.x+a*r.w;
        
        word.aim(ax,ay,true);
    }

    static void bounceWord(FloatingWord word, Room r)
    {
        if (word.inside(r.x,r.y,r.w,r.h))
            word.slow();
        else
            word.aim(r.x+r.w/2, r.y+r.h/2, false);
    }
    
}

