package apt.plan2d;

import java.util.*;
import java.awt.*;

/**
This class contains an algorithm for creating a pleasing
floorplan, given a certain number of rooms with
specified areas.
*/
class RandomFloorPlan implements FloorPlan
{
    RectangleAdjoiner adjoiner=new RectangleAdjoiner();
    long seed;

    public RandomFloorPlan()
    {
        seed=System.currentTimeMillis();
    }

    public RandomFloorPlan(long seed)
    {
        this.seed=seed;
    }
    
    public long getSeed()
    {
        return seed;
    }

    public Vector tile(Room[] items,
        double x, double y, double w, double h)
    {
        int n=items.length;
        Vector tour=new Vector();

        if (n==0) return tour;
        if (n==1)
        {
            items[0].setRect(x,y,w,h);
            tour.addElement(items[0]);
            return tour;
        }

        sort(items);

        Rectangle[] r=new Rectangle[n];
        int i;
        for(i=0; i<n; i++)
        {
            int s=(int)Math.sqrt(200*items[i].size);
            s=Math.max(4,s);
            r[i]=new Rectangle(0,0,s,s);
            items[i].clearChildren();
        }

        Rectangle bounds=new Rectangle(0,0,r[0].width,r[0].height);

        Random random=new Random(seed);

        for (i=1; i<n; i++)
        {
            int safecount=0;
            int b=0;
            do
            {
                b=(int)(i*random.nextDouble());
                Rectangle base=r[b];
                adjoiner.adjoin(base, r[i],
                             random.nextDouble(), .3*random.nextDouble());
            }
            while (lastOverlaps(r,i) && ++safecount<100);
            
            items[b].addChild(items[i]);
            //System.out.println(items[b]+" --> "+items[i]);
            
            bounds=bounds.union(r[i]);
        }

        // now rotate/scale if things don't fit in window.

        if (bounds.height>bounds.width)
        {
            for (i=0; i<n; i++)
                flip(r[i]);
            flip(bounds);

        }

        double scaleX=w/bounds.width;
        double scaleY=h/bounds.height;
        double scale=Math.min(scaleX, scaleY);
        double dx=-bounds.x;
        double dy=-bounds.y;


        // set room nodes to have these rectangle bounds

        for(i=0; i<n; i++)
        {
            Room m=items[i];
            m.x=x+scale*(r[i].x+dx);
            m.y=y+scale*(r[i].y+dy);
            m.w=scale*r[i].width;
            m.h=scale*r[i].height;

        }
        
        items[0].addTraversal(tour);
        return tour;
    }

    void flip(Rectangle r)
    {
        int t;
        t=r.x; r.x=r.y; r.y=t;
        t=r.width; r.width=r.height; r.height=t;
    }

    // does r[n] overlap any of r[0],...,r[n-1] ?
    boolean lastOverlaps(Rectangle[] r, int n)
    {
        for (int i=0; i<n; i++)
            if (adjoiner.overlap(r[n], r[i])>.3) return true;
        return false;
    }

    static double sum(Room[] s)
    {
        double sum=0;
        for (int i=0; i<s.length; i++)
            sum+=s[i].size;
        return sum;
    }

    static void sort(Room[] s) // put biggest first
    {
        int n=s.length;
        boolean outOfOrder=true;
        while (outOfOrder)
        {
            outOfOrder=false;
            for (int i=0; i<n-1; i++)
            {
                boolean wrong=(s[i].size<s[i+1].size);
                if (wrong)
                {
                    Room temp=s[i];
                    s[i]=s[i+1];
                    s[i+1]=temp;
                    outOfOrder=true;
                }
            }
        }
    }

}
