package apt.plan3d;

import apt.TypedWord;
import java.util.*;
import space.*;

/**
 *
 * The DisplayCommandBroadcaster is an intermediary between
 * the PlanEntryApplet and any SurfaceDisplayController
 * objects that it wishes to talk to.
 * <p>
 * This layer of abstraction allows us to easily
 * switch between a VRML output device and a native
 * Java test display.
 * <p>
 * A DisplayCommandBroadcaster takes the geometry
 * created by the Plan3D object (a Hashtable connecting
 * each FloatingWord in the apartment with a grid in 3D space)
 * and converts it to the kind of WordSurface data that
 * a surface display expects. It handles all updating and addition
 * of new words.
 *
 */
public class DisplayCommandBroadcaster
{
    // list of SurfaceDisplayControllers
    private Vector controllers=new Vector();

    // Hashtable of WordSurface objects.
    private Hashtable surfaces=new Hashtable();

    /**
     *
     * Add a new SurfaceDisplayController to the list
     * of controllers to listen to change events.
     *
     * @param s the new SurfaceDisplayController.
     *
     */
    public void addController(SurfaceDisplayController s)
    {
        controllers.addElement(s);
    }

    /**
     *
     * Update all controllers in response to a change.
     *
     * @param table a hashtable where the keys are FloatingWords and
     * the values are Point3D[][] grids.
     *
     */
    public void update(Hashtable table)
    {
        Enumeration k=table.keys();
        while (k.hasMoreElements())
        {
            TypedWord word=(TypedWord)k.nextElement();
            Point3D[][] grid=(Point3D[][])table.get(word);
            DefaultWordSurface surface=(DefaultWordSurface)surfaces.get(word);
            if (surface==null)
            {
                // add a new surface to controllers
                surface=new DefaultWordSurface(word.getWord(), grid);
                surfaces.put(word, surface);
                for (int i=0; i<controllers.size(); i++)
                    ((SurfaceDisplayController)controllers.elementAt(i)).
                        addSurface(surface);
            }
            else
                surface.setGrid(grid);


        }
        // Update display for all others.
        for (int i=0; i<controllers.size(); i++)
            ((SurfaceDisplayController)controllers.elementAt(i)).
                updateDisplay();

    }

    /**
     *
     * Alert all controllers that all previous data
     * should be cleared.
     *
     */
    public void clearSurfaces()
    {
        for (int i=0; i<controllers.size(); i++)
            ((SurfaceDisplayController)controllers.elementAt(i)).
                clearSurfaces();
    }

}