package apt.plan3d;

/**
 *
 * An interface to a class that controls some sort of
 * display capable of drawing WordSurfaces.
 * <P>
 * A SurfaceDisplayController should keep track of a list of
 * WordSurface objects. This list can be modified using the
 * addSurface() and clearSurfaces() methods.
 * <P>
 * When the updateDisplay() method is called, the
 * SurfaceDisplayController should update the display so that it
 * reflects the current list of surfaces. This will generally
 * involve three tasks:
 * <P>
 * 1. If any WordSurface have been added since the last update,
 * they should be added to the display.
 * <p>
 * 2. Other WordSurface may have changed their coordinates, so the
 * controller should make sure the display knows about their new
 * positions.
 * <p>
 * 3. The controller should make sure the display actually
 * redraws itself.
 */
public interface SurfaceDisplayController
{
    /**
     *
     * Add a new surface to the list of surfaces being tracked.
     *
     */
    public void addSurface(WordSurface surface);

    /**
     *
     * Remove all surfaces that are being tracked.
     *
     */
    public void clearSurfaces();

    /**
     *
     * Update the coordinates of all surfaces in the display,
     * and refresh the display.
     *
     */
    public void updateDisplay();
}
