package apt.servlet;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import com.bewitched.servlet.*;

/**
This is the main Apartment storage applet,
providing persistent data across sessions.
*/
public class Save extends HttpServlet
{ 
    private final String ERROR_SIGNAL="error";
    private final String OK_SIGNAL="ok";

    public void doGet(HttpServletRequest req, HttpServletResponse res)
                throws ServletException, IOException
    {
        res.setContentType("text/html");
        PrintWriter out=res.getWriter();

        try
        {
            // Get all info.
            String seed=req.getParameter("seed");
            String title=req.getParameter("title");
            String rooms=req.getParameter("rooms");
            String words=req.getParameter("words");
            String size=req.getParameter("size");
            String date=req.getParameter("date");
            String ratio=req.getParameter("ratio");

            // Check that nothing critical is null.
            notNull("seed", seed);
            notNull("title", title);
            notNull("rooms", rooms);
            notNull("words", words);
            notNull("size", size);
            notNull("date", date);
            notNull("ratio", ratio);

            // Write full info to a text file named after the seed.
            StringBuffer fullInfo=new StringBuffer();
            appendln(fullInfo, "#define title");
            appendln(fullInfo, title);
            appendln(fullInfo, "#define date");
            appendln(fullInfo, date);
            appendln(fullInfo, "#define ratio");
            appendln(fullInfo, ratio);
            appendln(fullInfo, "#define seed");
            appendln(fullInfo, seed);
            appendln(fullInfo, "#define words");
            appendln(fullInfo, words);
            TextFile apartmentFile=new TextFile(ApartmentPath.apartmentFile(seed));
            apartmentFile.write(fullInfo);


            // Add info to summary file.
            StringBuffer summaryInfo=new StringBuffer();
            appendln(summaryInfo, "===========");
            appendln(summaryInfo, "title\t"+title);
            appendln(summaryInfo, "size\t"+size);
            appendln(summaryInfo, "seed\t"+seed);
            appendln(summaryInfo, "ratio\t"+ratio);
            appendln(summaryInfo, "date\t"+date);
            appendln(summaryInfo, rooms);
            TextFile indexFile=new TextFile(ApartmentPath.indexFile());
            indexFile.append(summaryInfo);


            // Send out info that all is OK.
            out.println(OK_SIGNAL);

        }
        catch (Exception e)
        {
            out.println(ERROR_SIGNAL);
            out.println(e);
        }

    }


    public void doPost(HttpServletRequest req, HttpServletResponse res)
                throws ServletException, IOException
    {
        doGet(req, res);
    }

    private void notNull(String what, Object o)
    {
        if (o==null) throw new NullPointerException(what+" was null!");
    }

    private void appendln(StringBuffer b, String s)
    {
        b.append(s);
        b.append(TextFile.CRLF);
    }
}
