package apt.vrml;

import apt.plan3d.WordSurface;
import space.Point3D;
import java.io.*;
import java.util.*;
import java.text.*;

public class AtLastItsTheVRMLConverter
	implements VRMLConverter
{
	static final double SCALE = .05;
	static final DecimalFormat f = new DecimalFormat("0.00");

	String baseURL;
	String basePath;

	public AtLastItsTheVRMLConverter(String baseURL, String basePath)
	{
		this.baseURL = baseURL;
		this.basePath = basePath;
	}

	public AtLastItsTheVRMLConverter()
		throws IOException
	{
		Properties props = new Properties();
		try {
			props.load(this.getClass().getResourceAsStream("VRMLConverter.properties"));
		} catch (IOException e) {
			System.err.println(e);
			throw new IOException(e.toString());
		}
		baseURL = props.getProperty("baseURL");
		basePath = props.getProperty("basePath");
		System.err.println("baseURL = " + baseURL);
		System.err.println("basePath = " + basePath);
	}


	static class Orientation
	{
		public Point3D axis;
		public double rads;

		public Orientation(Point3D axis, double rads)
		{
			this.axis = axis;
			this.rads = rads;
		}
	}

    public void writeToVRML(RoomInfo[] rooms,
                            int[] tour,
                            Writer w_out)
		throws IOException
	{
		Vector /* Point3D */ tourPoints = new Vector();
		Vector /* Point3D */ tourOrientations = new Vector();
		
		Point3D baseOrientation = new Point3D(0, 0, 1);

		Point3D tourStart = null;

		StringBuffer sb = new StringBuffer(16384); // 16K buffer
		for (int ri = 0; ri < rooms.length; ri++) {
			sb.append("Transform { children [\n");

			Point3D[] c = rooms[ri].getCorners();
			NurbsInfo.transformMartinPoints(c);
			
			//for (int i = 0; i < 3; i++)
				sb.append(makeSketchedBox(c));

			Point3D roomCenter = NurbsInfo.findCenter(c);
			if (tourStart == null) tourStart = roomCenter;

			WordSurface[] surfaces = rooms[ri].getWordSurfaces();
			for (int i = 0; i < surfaces.length; i++) {
				Point3D[][] grid = surfaces[i].getGrid();
				String word = surfaces[i].getWord();
				NurbsInfo nurbs = new NurbsInfo(grid);
				String textureURL = getTextureURL(word);
				String soundURL = getSoundURL(word);

				
				Point3D orientation = roomCenter.minus(nurbs.center).unit();
				Point3D axis = orientation.cross(baseOrientation);
				double theta = Math.acos(orientation.dot(baseOrientation));
				orientation.scale(5);
				Point3D position = roomCenter;//.minus(orientation);
				//position.y *= .2;
				Orientation or = new Orientation(axis, theta);
				tourPoints.addElement(position);
				tourOrientations.addElement(or);

				sb.append("W { nurbsCoords [\n");
				for (int j = 0; j < nurbs.grid.length; j++) {
					for (int k = 0; k < nurbs.grid[j].length; k++) {
						Point3D p = nurbs.grid[j][k];
						sb.append(f.format(p.x) + " " + f.format(p.y) + " " 
								  + f.format(p.z) + ",");
					}
				}
				sb.append("]\n");
				sb.append(" nurbsTexture \"" + textureURL + "\"\n");
				sb.append("}\n");
				
				sb.append("S { url \"" + soundURL + "\"\n");
				sb.append("translation "
							+ f.format(nurbs.center.x) + " "
							+ f.format(nurbs.center.y) + " "
							+ f.format(nurbs.center.z) + "\n");
				sb.append("}\n");
			}
			sb.append("]}\n");
		}
		tourPoints.addElement(tourStart);
		tourOrientations.addElement(new Orientation(new Point3D(0, 1, 0), Math.PI/2));
		
		PrintWriter out = new PrintWriter(new BufferedWriter(w_out));
		try {
			BufferedReader template = new BufferedReader(new FileReader
				(basePath + "vrml/ApartmentTemplate.wrl"));
			
			String line;
			while ((line = template.readLine()) != null) {
				out.println(line);
				if (line.indexOf("ROOMPOINTS") > -1)
					out.println( getRoomCoords() );
				else if (line.indexOf("ROOMINDEX") > -1)
					out.println( getRoomIndex() );
				else if (line.indexOf("TOURTIME") > -1)
					out.println( 6 * tourPoints.size() );
				else if (line.indexOf("TOURKEY") > -1)
					out.println( getTourKey(tourPoints) );
				else if (line.indexOf("TOURPOINTS") > -1)
					out.println( getTourPoints(tourPoints) );
				else if (line.indexOf("TOUROKEY") > -1)
					out.println( getTourKey(tourOrientations) );
				else if (line.indexOf("TOUROPOINTS") > -1)
					out.println( getTourOrientations(tourOrientations) );
			}
		} catch (IOException e) {
			System.err.println(e);
			throw new IOException(e.toString());
		}
		out.println(sb.toString());
		
		out.close();
	}
	
	
	static private final double fudge(double d)
	{
		double s = .1;

		return d - s + Math.random() * 2 * s;
	}

	private StringBuffer skb = new StringBuffer();
	private String makeSketchedBox(Point3D[] c)
	{
		
		skb.setLength(0);
		skb.append("BLUEBOX { corners [");
		skb.append(f.format(fudge(c[0].x)) + " " 
				  + f.format(fudge(c[0].y)) + " " 
				  + f.format(fudge(c[0].z)) + ",");
		skb.append(f.format(fudge(c[1].x)) + " " 
				  + f.format(fudge(c[0].y)) + " " 
				  + f.format(fudge(c[0].z)) + ",");
		skb.append(f.format(fudge(c[1].x)) + " " 
				  + f.format(fudge(c[1].y)) + " " 
				  + f.format(fudge(c[0].z)) + ",");
		skb.append(f.format(fudge(c[0].x)) + " " 
				  + f.format(fudge(c[1].y)) + " " 
				  + f.format(fudge(c[0].z)) + ",");
		skb.append(f.format(fudge(c[0].x)) + " " 
				  + f.format(fudge(c[0].y)) + " " 
				  + f.format(fudge(c[1].z)) + ",");
		skb.append(f.format(fudge(c[1].x)) + " " 
				  + f.format(fudge(c[0].y)) + " " 
				  + f.format(fudge(c[1].z)) + ",");
		skb.append(f.format(fudge(c[1].x)) + " " 
				  + f.format(fudge(c[1].y)) + " " 
				  + f.format(fudge(c[1].z)) + ",");
		skb.append(f.format(fudge(c[0].x)) + " " 
				  + f.format(fudge(c[1].y)) + " " 
				  + f.format(fudge(c[1].z)) + ",");
		skb.append("]}");
		return skb.toString();
	}

	private String getRoomCoords()
	{
		return "21.7 0 11.7, 21.7 0 -8.33,11.7 0 -8.33,11.7 0 -18.3, -8.33 0 -18.3,-8.33 0 -8.33,  -18.3 0 -8.33, -18.3 0 1.67, -8.33 0 1.67, -8.33 0 21.7,1.67 0 21.7, 1.67 0 11.7, 21.7 0 11.7, 21.7 0 -8.33, 11.7 0 -8.33,11.7 0 -18.3, -8.33 0 -18.3, -8.33 0 -8.33, -18.3 0 -8.33, -18.3 0 1.67,	-8.33 0 1.67, -8.33 0 21.7, 1.67 0 21.7, 1.67 0 11.7";
	}

	private String getRoomIndex()
	{
		return "11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0, -1, 21, 22, 23, 12, 13, 14, 15, 16, 17, 18, 19, 20, -1";
	}

	private String getTourKey(Vector v)
	{
		StringBuffer sb = new StringBuffer();
		double delta = 1.0 / v.size();
		for (int i = 0; i < v.size(); i++)
			sb.append(f.format(delta * i)).append(", ");
		return sb.toString();
	}

	private String getTourPoints(Vector v)
	{
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < v.size(); i++) {
			Point3D p = (Point3D)v.elementAt(i);
			sb.append(f.format(p.x)).append(" ").append(f.format(p.y)).append(" ");
			sb.append(f.format(p.z)).append(",\n");
		}
		return sb.toString();
	}

	private String getTourOrientations(Vector v)
	{
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < v.size(); i++) {
			Orientation o = (Orientation)v.elementAt(i);
			sb.append(f.format(0/*o.axis.x*/)).append(" ");
			sb.append(f.format(o.axis.y)).append(" ");
			sb.append(f.format(0/*o.axis.z*/)).append(" ");
			sb.append(o.rads).append(",\n");
		}
		return sb.toString();
	}

	private String getTextureURL(String word)
	{
		final String prefix = word + "_";
		String defaultURL = baseURL + "vrml/debug_texture.jpg";
		String letter = word.substring(0, 1);
		File textureDirectory = new File(basePath + "vrml/textures/" + letter + "/");
		String[] matches = textureDirectory.list(new FilenameFilter () {
				public boolean accept(File dir, String name) {
					return name.startsWith(prefix);
				}
			});
		if (matches == null || matches.length == 0)
			return defaultURL;
		int index = (int)(Math.random() * matches.length);
		return baseURL + "vrml/textures/" + letter + "/" + matches[index];
	}

	private String getSoundURL(String word)
	{
		String wavfilename = word + ".wav";
		String letter = word.substring(0, 1);
		File wavfile = new File(basePath + "vrml/audio/" + letter + "/" + wavfilename);
		if (wavfile.exists())
			return baseURL + "vrml/audio/" + letter + "/" + wavfilename;
		else
			return "";
	}

	public static void main(String[] args)
		throws IOException
	{
		File in = new File(args[0]);
		RoomInfo[] rooms = RoomIO.exampleInfo(in);
		FileWriter file = new FileWriter("out.wrl");
		VRMLConverter vc = new AtLastItsTheVRMLConverter
			("http://www.turbulence.org/Works/apartment/", "/home/newradio/Works/apartment/");
		vc.writeToVRML(rooms, null, file);
		file.close();
	}
}
