package apt.vrml;

import space.*;
import java.util.Vector;

public class NurbsInfo
{
	static final double SCALE = .05;

	public Point3D[][] grid  = null;
	public Point3D center    = null;
	public Point3D direction = null;
	public double magnitude;

	static public void transformMartinPoints(Point3D[] points)
	{
		for (int i = 0; i < points.length; i++) {
			points[i].x *= SCALE;
			points[i].y *= SCALE;
			points[i].z *= SCALE;
		}
	}

	static public void transformMartinGrid(Point3D[][] grid)
	{
		for (int j = 0; j < grid.length; j++)
			transformMartinPoints(grid[j]);
	}

	static public Point3D findCenter(Point3D[] points)
	{
		double sx = 0;
		double sy = 0;
		double sz = 0;
		for (int j = 0; j < points.length; j++) {
			sx += points[j].x;
			sy += points[j].y;
			sz += points[j].z;
		}
		return new Point3D( sx / points.length, sy / points.length, sz / points.length );
	}

	static public Point3D findCenter(Vector points)
	{
		Point3D[] apoints = new Point3D[points.size()];
		points.copyInto(apoints);
		return findCenter(apoints);
	}

	public NurbsInfo(Point3D[][] martinGrid) 
	{
		if (martinGrid.length != martinGrid[0].length)
			throw new RuntimeException
				("NurbsInfo can only deal with square patches!");

		grid = martinGrid;
		transformMartinGrid(grid);

		Vector allPoints = new Vector();
		Vector centerPoints = new Vector();
		int n = grid.length;
		boolean even = n % 2 == 0;
		for (int j = 0; j < grid.length; j++)
		    for (int k = 0; k < grid.length; k++) {
				allPoints.addElement( grid[j][k] );
				if (   ((j == n/2) || (even && (j == n/2 - 1)))
					   &&
					   ((k == n/2) || (even && (k == n/2 - 1))))
					centerPoints.addElement( grid[j][k] );
			}
		
		center = findCenter(allPoints);
		Point3D c = findCenter(centerPoints);
		direction = new Point3D(center.x - c.x, center.y - c.y, center.z - c.z);
		magnitude = Math.sqrt(Math.pow(direction.x, 2) +
							  Math.pow(direction.y, 2) +
							  Math.pow(direction.z, 2));
		direction.scale(1.0 / magnitude);
		tweakControlPoints();
	}
	
	private void tweakCorner(Point3D c, Point3D e1, Point3D e2)
	{
		double s = 1 * magnitude;
		double ef = .5;

		double x = -s + (Math.random() * 2 * s);
		double y = -s + (Math.random() * 2 * s);
		double z = -s + (Math.random() * 2 * s);

		c.x += x;
		c.y += y;
		c.z += z;
		e1.x += x * ef;
		e1.y += y * ef;
		e1.z += z * ef;
		e2.x += x * ef;
		e2.y += y * ef;
		e2.z += z * ef;
	}

	private void tweakControlPoints()
	{
		int n = grid.length;
		
		tweakCorner( grid[0][0], grid[0][1], grid[1][0] );
		tweakCorner( grid[0][grid.length - 1], grid[0][grid.length - 2],
					 grid[1][grid.length - 1]);
		tweakCorner( grid[grid.length - 1][0], grid[grid.length - 2][0],
					 grid[grid.length - 1][1]);
		tweakCorner( grid[grid.length - 1][grid.length - 1], 
					 grid[grid.length - 1][grid.length - 2],
					 grid[grid.length - 2][grid.length - 1]);

		boolean even = n % 2 == 0;
		for (int j = 0; j < n; j++)
			for (int k = 0; k < n; k++) {
				if (((j == n/2) || (even && (j == n/2 - 1)))
					&&
					((k == n/2) || (even && (k == n/2 - 1)))) 
					{
						/*
						  Pull the central point(s) out farther to exaggerate
						  the curve
						*/
						grid[j][k].x -= magnitude * direction.x * (2 + Math.random());
						grid[j][k].y -= magnitude * direction.y * (2 + Math.random());
						grid[j][k].z -= magnitude * direction.z * (2 + Math.random());
					}
			}
		
	}
		
}
