package english.syntax;

import english.words.Word;

/**
 *
 * Code for deciding whether two words can be paired
 * together with correct syntax. For instance,
 * "cats" and "run" go together syntactically,
 * but "cat" and "run" do not.
 *
 */
public abstract class WordMatcher
{
    /** 
     *
     * Decide whether two words can occur next to each other
     * in a grammatically correct-seeming way.
     *
     * @param w1 the first word.
     * @param w2 the second word.
     * @return a String representing the pairing. If
     * the String is null, then the two words cannot be paired.
     *
     */
    public static String canPair(Word w1, Word w2)
    {
        if (w1==null || w2==null)
            return null;
        /*if (w1.word.equalsIgnoreCase("I"))
        {
            System.out.println(w1+"?"+w2);
        }*/

        if (!w2.proper && Character.isUpperCase(w2.word.charAt(0)))
            return null;
            
        if (
            w1.verb && !w1.intrans && w2.noun &&
                (!w2.pronoun || w2.directObj) ||
            w1.verb && w2.adv  ||
            w1.adj  && w2.noun && !w2.proper)
            return w1+" "+w2;

        if (w1.noun && !w1.directObj && w2.verb)
        {
            FeatureHolder nf=FeatureClassifier.classifyNoun(w1.word);
            FeatureHolder vf=FeatureClassifier.classifyVerb(w2.word);
            if (nf.hasMatch(vf))
                return w1+" "+w2;
        }
        return null;
    }
}